@@ -1,10 +1,12 @@
+import webbrowser
+
 from kivy.utils import platform
 from kivy.logger import Logger
 
 if platform == "android":
     try:
         from kivy.core.window import Window
-        from jnius import autoclass, cast
+        from jnius import autoclass, cast, JavaException
         from android.runnable import run_on_ui_thread
 
         AndroidActivity = autoclass('android.app.Activity')
@@ -18,6 +20,8 @@ if platform == "android":
         WindowManager = autoclass('android.view.WindowManager$LayoutParams')
         Intent = autoclass('android.content.Intent')
         Provider = autoclass('android.provider.Settings')
+        URLConnection = autoclass("java.net.URLConnection")
+        Toast = autoclass('android.widget.Toast')
         Uri = autoclass('android.net.Uri')
         Locale = autoclass('java.util.Locale')
         ConnectivityManager = autoclass('android.net.ConnectivityManager')
@@ -34,13 +38,30 @@ if platform == "android":
         StrictMode = autoclass('android.os.StrictMode')
         MediaPlayer = autoclass('android.media.MediaPlayer')
         AudioManager = autoclass('android.media.AudioManager')
-        rctx = autoclass('android.graphics.Rect')()
+        Rect = autoclass('android.graphics.Rect')()
         StatFs = autoclass("android.os.StatFs")
         MemoryInfo = autoclass('android.app.ActivityManager$MemoryInfo')
         BatteryManager = autoclass("android.os.BatteryManager")
         IntentFilter = autoclass("android.content.IntentFilter")
         Point = autoclass("android.graphics.Point")
 
+        packages = {
+            "whatsapp": "com.whatsapp",
+            "facebook": "com.facebook.katana",
+            "facebookLite": "com.facebook.lite",
+            "oldFacebook": "com.facebook.android",
+            "linkedin": "com.linkedin.android",
+            "fbMessenger": "com.facebook.orca",
+            "fbMessengerLite": "com.facebook.mlite",
+            "tiktok": "com.zhiliaoapp.musically",
+            "tiktokLite": "com.zhiliaoapp.musically.go",
+            "twitter": "com.twitter.android",
+            "twitterLite": "com.twitter.android.lite",
+            "telegram": "org.telegram.messenger",
+            "telegramX": "org.thunderdog.challegram",
+            "snapchat": "com.snapchat.android"
+        }
+
     except BaseException:
         Logger.error(
             "Kvdroid: Cannot load classes by Pyjnius. Make sure requirements installed"
@@ -51,13 +72,13 @@ if platform == "android":
         try:
             decor_view = activity.getWindow().getDecorView()
             height = activity.getWindowManager().getDefaultDisplay().getHeight()
-            decor_view.getWindowVisibleDisplayFrame(rctx)
-            return height - rctx.bottom
+            decor_view.getWindowVisibleDisplayFrame(Rect)
+            return height - Rect.bottom
         except:
             return 0
 
 
-    def device_info(yaz,convert = False):
+    def device_info(text,convert = False):
         bm = activity.getSystemService(Context.BATTERY_SERVICE)        
         count = bm.getIntProperty(BatteryManager.BATTERY_PROPERTY_CHARGE_COUNTER)
         cap = bm.getIntProperty(BatteryManager.BATTERY_PROPERTY_CAPACITY)        
@@ -150,7 +171,8 @@ if platform == "android":
             'bat_voltage': float(intent.getIntExtra(BatteryManager.EXTRA_VOLTAGE,0) * 0.001),
             'bat_technology': intent.getStringExtra(BatteryManager.EXTRA_TECHNOLOGY)
         }
-        return os[yaz]
+        return os[text]
+
 
     @run_on_ui_thread
     def immersive_mode(immerse=None):
@@ -241,16 +263,28 @@ if platform == "android":
             Runtime.getRuntime().exit(0)
 
 
-    def share_text(text, title='Share'):
+    def share_text(text, title='Share', chooser=False, app_package=None, call_playstore=True, error_msg=""):
         intent = Intent()
         intent.setAction(Intent.ACTION_SEND)
         intent.putExtra(Intent.EXTRA_TEXT, String(str(text)))
         intent.setType("text/plain")
-        chooser = Intent.createChooser(intent, String(title))
-        activity.startActivity(chooser)
+        if app_package:
+            app_package = packages[app_package] if app_package in packages else None
+            try:
+                intent.setPackage(String(app_package))
+            except JavaException:
+                if call_playstore:
+                    webbrowser.open(f"http://play.google.com/store/apps/details?id={app_package}")
+                toast(error_msg) if error_msg else Logger.error("Kvdroid: Specified Application is unavailable")
+                return
+        if chooser:
+            chooser = Intent.createChooser(intent, String(title))
+            activity.startActivity(chooser)
+        else:
+            activity.startActivity(intent)
 
 
-    def share_file(path, title='Share', chooser=False, app_package=None):
+    def share_file(path, title='Share', chooser=True, app_package=None, call_playstore=True, error_msg=""):
         path = str(path)
         StrictMode.disableDeathOnFileUriExposure()
         shareIntent = Intent(Intent.ACTION_SEND)
@@ -259,21 +293,26 @@ if platform == "android":
         uri = Uri.fromFile(imageFile)
         parcelable = cast('android.os.Parcelable', uri)
         shareIntent.putExtra(Intent.EXTRA_STREAM, parcelable)
-        currentActivity = cast('android.app.Activity', activity)
 
-        if not app_package:
+        if app_package:
+            app_package = packages[app_package] if app_package in packages else None
             try:
                 shareIntent.setPackage(String(app_package))
-            except Exception as e:
-                Logger.error(
-                    f"Kvdroid: Specified Application is unavailable, {e}"
-                )
+            except JavaException:
+                if call_playstore:
+                    webbrowser.open(f"http://play.google.com/store/apps/details?id={app_package}")
+                toast(error_msg) if error_msg else Logger.error("Kvdroid: Specified Application is unavailable")
+                return
 
         if chooser:
             chooser = Intent.createChooser(shareIntent, String(title))
-            currentActivity.startActivity(chooser)
+            activity.startActivity(chooser)
         else:
-            currentActivity.startActivity(shareIntent)
+            activity.startActivity(shareIntent)
+
+    def mime_type(file_path):
+        return URLConnection.guessContentTypeFromName(file_path)
+
 
 
 else:
