@@ -1,5 +1,5 @@
-from kvdroid import activity
-
+from kvdroid import activity, Point
+import re
 
 class Metrics(object):
     config = activity.getResources().getConfiguration()
@@ -22,6 +22,11 @@ class Metrics(object):
             return "portrait"
         else:
             return "landscape"
-
+            
+    def resolution(self):
+        point = Point()
+        activity.getWindowManager().getDefaultDisplay().getRealSize(point)
+        size = re.findall("\d+",point.toString())
+        return size[1]+"x"+size[0]
 
 screen = Metrics()
